from packaging.version import Version
from rich.console import Console

from jupyter_deploy import cmd_utils


def check_terraform_installation(min_version: Version | None = None) -> bool:
    """Shell out to verify terraform installation, return True if valid."""

    installed, str_version, error_msg = cmd_utils.check_executable_installation(
        executable_name="terraform",
    )
    console = Console()

    if not installed:
        console.print(
            "This operation requires [bold]terraform[/] to be installed in your system.\n"
            "Got the following error when verifying installation:"
        )
        console.print(error_msg, style="red")
        console.print(
            "Refer to the installation guide: "
            "https://developer.hashicorp.com/terraform/tutorials/aws-get-started/install-cli",
        )
        return False

    if min_version:
        if not str_version:
            console.print(
                "Current version of [bold]terraform[/] not found, cannot perform minimum version check.", style="red"
            )
            return False

        current_version = Version(str_version)
        if current_version >= min_version:
            console.print(":white_check_mark: Valid [bold]terraform[/] installation detected.")
            return True
        else:
            console.print(
                f"This operation requires minimum [bold]terraform[/] version: {min_version}\n"
                f"Found version: {current_version}"
            )
            console.print(f"Upgrade [bold]terraform[/] at least to version: {min_version}.", style="yellow")
            return False

    console.print(":white_check_mark: Valid [bold]terraform[/] installation detected.")
    return True



import json
import os
from collections.abc import Generator
from pathlib import Path
from unittest.mock import patch

import pytest

from jupyter_deploy.engine.enum import EngineType
from jupyter_deploy.handlers.project.open_handler import OpenHandler
from jupyter_deploy.manifest import JupyterDeployManifestV1

# Define the constant locally since it was removed from tf_constants
TF_STATEFILE = "terraform.tfstate"


@pytest.fixture
def mock_cwd(tmp_path: Path) -> Generator[Path, None, None]:
    """Create a temporary directory and set it as the current working directory."""
    original_dir = os.getcwd()
    os.chdir(tmp_path)
    yield tmp_path
    os.chdir(original_dir)


@pytest.fixture
def mock_manifest() -> JupyterDeployManifestV1:
    """Create a mock manifest."""
    return JupyterDeployManifestV1(
        **{  # type: ignore
            "schema_version": 1,
            "template": {
                "name": "mock-template-name",
                "engine": "terraform",
                "version": "1.0.0",
            },
        }
    )


@pytest.fixture
def mock_tfstate(mock_cwd: Path) -> Path:
    """Create a mock terraform.tfstate file with a jupyter_url output."""
    tfstate_content = {
        "version": 4,
        "outputs": {"jupyter_url": {"value": "https://example.com/jupyter", "type": "string"}},
    }
    tfstate_path = mock_cwd / TF_STATEFILE
    with open(tfstate_path, "w") as f:
        json.dump(tfstate_content, f)
    return tfstate_path


class TestOpenHandler:
    def test_init(self, mock_manifest: JupyterDeployManifestV1) -> None:
        """Test that the OpenHandler initializes correctly."""
        with patch("jupyter_deploy.handlers.base_project_handler.retrieve_project_manifest") as mock_retrieve_manifest:
            mock_retrieve_manifest.return_value = mock_manifest
            handler = OpenHandler()
            assert handler._handler is not None
            assert handler.engine == EngineType.TERRAFORM
            assert handler.project_manifest == mock_manifest

    def test_open_url_success(self, mock_manifest: JupyterDeployManifestV1) -> None:
        """Test that open_url opens the URL in a web browser, and outputs the URL and cookies help message."""
        with patch("jupyter_deploy.handlers.base_project_handler.retrieve_project_manifest") as mock_retrieve_manifest:
            mock_retrieve_manifest.return_value = mock_manifest
            handler = OpenHandler()
            with (
                patch("webbrowser.open", return_value=True) as mock_open,
                patch.object(handler.console, "print") as mock_print,
            ):
                handler.open_url("https://example.com/jupyter")
                mock_open.assert_called_once_with("https://example.com/jupyter", new=2)
                assert mock_print.call_count == 2
                assert "Opening Jupyter" in mock_print.call_args_list[0][0][0]
                assert "cookies" in mock_print.call_args_list[1][0][0]

    def test_open_url_empty(self, mock_manifest: JupyterDeployManifestV1) -> None:
        """Test that open_url doesn't do anything when the URL is empty."""
        with patch("jupyter_deploy.handlers.base_project_handler.retrieve_project_manifest") as mock_retrieve_manifest:
            mock_retrieve_manifest.return_value = mock_manifest
            handler = OpenHandler()
            with patch("webbrowser.open") as mock_open, patch.object(handler.console, "print") as mock_print:
                handler.open_url("")
                mock_open.assert_not_called()
                mock_print.assert_not_called()

    def test_open_url_error(self, mock_manifest: JupyterDeployManifestV1) -> None:
        """Test that open_url handles errors when opening the URL."""
        with patch("jupyter_deploy.handlers.base_project_handler.retrieve_project_manifest") as mock_retrieve_manifest:
            mock_retrieve_manifest.return_value = mock_manifest
            handler = OpenHandler()
            with (
                patch("webbrowser.open", return_value=False) as mock_open,
                patch.object(handler.console, "print") as mock_print,
            ):
                handler.open_url("https://example.com/jupyter")
                mock_open.assert_called_once_with("https://example.com/jupyter", new=2)
                assert mock_print.call_count == 3
                assert "Failed to open URL" in mock_print.call_args_list[2][0][0]

    def test_open_url_insecure(self, mock_manifest: JupyterDeployManifestV1) -> None:
        """Test that open_url doesn't open non-HTTPS urls."""
        with patch("jupyter_deploy.handlers.base_project_handler.retrieve_project_manifest") as mock_retrieve_manifest:
            mock_retrieve_manifest.return_value = mock_manifest
            handler = OpenHandler()
            with (
                patch("webbrowser.open") as mock_open,
                patch.object(handler.console, "print") as mock_print,
            ):
                handler.open_url("http://example.com/jupyter")
                mock_open.assert_not_called()
                mock_print.assert_called_once()
                assert "Insecure URL detected" in mock_print.call_args[0][0]



import unittest
from pathlib import Path
from unittest.mock import Mock, patch

from jupyter_deploy.engine.terraform import tf_vardefs, tf_varfiles


class TestParseVariablesDotTfContent(unittest.TestCase):
    variables_tf_content: str

    @classmethod
    def setUpClass(cls) -> None:
        mock_variables_path = Path(__file__).parent / "mock_variables.tf"
        with open(mock_variables_path) as f:
            cls.variables_tf_content = f.read()

    def test_can_parse_empty_content(self) -> None:
        result = tf_varfiles.parse_variables_dot_tf_content("")
        self.assertEqual({}, result)

    @patch("hcl2.loads")
    def test_calls_hcl2_loads(self, mock_loads: Mock) -> None:
        mock_loads.return_value = {"variable": []}
        tf_varfiles.parse_variables_dot_tf_content(self.variables_tf_content)
        mock_loads.assert_called_once_with(self.variables_tf_content)

    @patch("hcl2.loads")
    def test_parses_valid_variables_tf(self, mock_loads: Mock) -> None:
        # Create a mock return value that simulates the parsed variables.tf content
        mock_parsed = {
            "variable": [
                {
                    "some_string_value": {
                        "description": "For example the instance type.\n\nRecommended: t3.medium",
                        "type": "string",
                    }
                },
                {
                    "some_int_value": {
                        "description": "For example the size of the disk in GB.\n\nRecommended: 30",
                        "type": "number",
                    }
                },
            ]
        }
        mock_loads.return_value = mock_parsed
        result = tf_varfiles.parse_variables_dot_tf_content(self.variables_tf_content)

        # Verify the result contains the expected variables
        self.assertIn("some_string_value", result)
        self.assertIn("some_int_value", result)

        # Verify the variable definitions have the correct types
        self.assertIsInstance(result["some_string_value"], tf_vardefs.TerraformStrVariableDefinition)
        self.assertIsInstance(result["some_int_value"], tf_vardefs.TerraformNumberVariableDefinition)

        # Verify the descriptions are correctly parsed
        self.assertEqual(
            "For example the instance type.\n\nRecommended: t3.medium", result["some_string_value"].description
        )
        self.assertEqual(
            "For example the size of the disk in GB.\n\nRecommended: 30", result["some_int_value"].description
        )

    def test_parsing_actual_variables_tf_file_works(self) -> None:
        content = self.variables_tf_content
        result = tf_varfiles.parse_variables_dot_tf_content(content)

        # Verify all expected variables are present
        self.assertIn("some_string_value", result)
        self.assertIn("some_int_value", result)
        self.assertIn("some_float_value", result)
        self.assertIn("some_string_value_with_condition", result)
        self.assertIn("some_secret", result)

        # Verify variables have the correct types
        self.assertIsInstance(result["some_string_value"], tf_vardefs.TerraformStrVariableDefinition)
        self.assertIsInstance(result["some_int_value"], tf_vardefs.TerraformNumberVariableDefinition)
        self.assertIsInstance(result["some_float_value"], tf_vardefs.TerraformNumberVariableDefinition)
        self.assertIsInstance(result["some_string_value_with_condition"], tf_vardefs.TerraformStrVariableDefinition)
        self.assertIsInstance(result["some_secret"], tf_vardefs.TerraformStrVariableDefinition)
        self.assertIsInstance(result["some_list_of_string"], tf_vardefs.TerraformListOfStrVariableDefinition)
        self.assertIsInstance(result["some_map_of_sring"], tf_vardefs.TerraformMapOfStrVariableDefinition)

        # Verify descriptions are correctly parsed
        self.assertIn("Recommended: t3.medium", result["some_string_value"].description)
        self.assertIn("For example the size of the disk in GB.", result["some_int_value"].description)
        self.assertIn("Recommended: 0.9", result["some_float_value"].description)

        # Verify sensitive flag is set correctly
        self.assertTrue(result["some_secret"].sensitive)
        self.assertFalse(result["some_string_value"].sensitive)


class TestParseDotTfvarsContentAndAddDefaults(unittest.TestCase):
    variables_tf_content: str
    tfvars_content: str

    @classmethod
    def setUpClass(cls) -> None:
        mock_variables_path = Path(__file__).parent / "mock_variables.tf"
        with open(mock_variables_path) as f:
            cls.variables_tf_content = f.read()

        mock_tfvars_path = Path(__file__).parent / "mock_defaults.tfvars"
        with open(mock_tfvars_path) as f:
            cls.tfvars_content = f.read()

    def test_can_parse_empty_content(self) -> None:
        variable_defs: dict[str, tf_vardefs.TerraformVariableDefinition] = {}
        tf_varfiles.parse_dot_tfvars_content_and_add_defaults("", variable_defs)
        self.assertEqual({}, variable_defs)  # Should remain empty

    @patch("hcl2.loads")
    def test_calls_hcl2_loads(self, mock_loads: Mock) -> None:
        mock_loads.return_value = {}
        variable_defs: dict = {}
        tf_varfiles.parse_dot_tfvars_content_and_add_defaults(self.tfvars_content, variable_defs)
        mock_loads.assert_called_once_with(self.tfvars_content)

    @patch("hcl2.loads")
    def test_parse_valid_tfvars_content_and_add(self, mock_loads: Mock) -> None:
        # Create mock variable definitions
        string_var_def = tf_vardefs.TerraformStrVariableDefinition(
            variable_name="some_string_value", description="A string variable"
        )
        int_var_def = tf_vardefs.TerraformNumberVariableDefinition(
            variable_name="some_int_value", description="An int variable"
        )

        variable_defs: dict[str, tf_vardefs.TerraformVariableDefinition] = {
            "some_string_value": string_var_def,
            "some_int_value": int_var_def,
        }

        # Create mock tfvars content
        mock_tfvars = {"some_string_value": "t3.medium", "some_int_value": 30}

        mock_loads.return_value = mock_tfvars
        tf_varfiles.parse_dot_tfvars_content_and_add_defaults(self.tfvars_content, variable_defs)

        # Verify defaults were added to the variable definitions
        self.assertEqual("t3.medium", variable_defs["some_string_value"].default)
        self.assertEqual(30, variable_defs["some_int_value"].default)

    @patch("hcl2.loads")
    def test_skips_vars_not_found_in_variables_tf(self, mock_loads: Mock) -> None:
        # Create mock variable definitions
        string_var_def = tf_vardefs.TerraformStrVariableDefinition(
            variable_name="some_string_value", description="A string variable"
        )

        variable_defs: dict[str, tf_vardefs.TerraformVariableDefinition] = {"some_string_value": string_var_def}

        # Create mock tfvars content with an unrecognized variable
        mock_tfvars = {"some_string_value": "t3.medium", "some_unrecognized_value": "manually-added"}

        mock_loads.return_value = mock_tfvars
        tf_varfiles.parse_dot_tfvars_content_and_add_defaults(self.tfvars_content, variable_defs)

        # Verify only the recognized variable was updated
        self.assertEqual("t3.medium", variable_defs["some_string_value"].default)
        self.assertEqual(1, len(variable_defs))  # No new variables should be added

    @patch("hcl2.loads")
    def test_skips_sensitive_variables(self, mock_loads: Mock) -> None:
        # Create mock variable definitions including a sensitive one
        string_var_def = tf_vardefs.TerraformStrVariableDefinition(
            variable_name="some_string_value", description="A string variable"
        )
        sensitive_var_def = tf_vardefs.TerraformStrVariableDefinition(
            variable_name="some_secret", description="A sensitive variable", sensitive=True
        )

        variable_defs: dict[str, tf_vardefs.TerraformVariableDefinition] = {
            "some_string_value": string_var_def,
            "some_secret": sensitive_var_def,
        }

        # Create mock tfvars content with a sensitive variable
        mock_tfvars = {"some_string_value": "t3.medium", "some_secret": "i-should-not-be-here"}

        mock_loads.return_value = mock_tfvars
        tf_varfiles.parse_dot_tfvars_content_and_add_defaults(self.tfvars_content, variable_defs)

        # Verify only the non-sensitive variable was updated
        self.assertEqual("t3.medium", variable_defs["some_string_value"].default)
        self.assertIsNone(variable_defs["some_secret"].default)  # Sensitive variable should not be updated

    def test_parsing_actual_variables_tf_and_defaults_tfvars_files_works(self) -> None:
        parsed_variable_defs = tf_varfiles.parse_variables_dot_tf_content(self.variables_tf_content)
        tf_varfiles.parse_dot_tfvars_content_and_add_defaults(self.tfvars_content, parsed_variable_defs)

        # Verify defaults were added to the variable definitions
        self.assertEqual("t3.medium", parsed_variable_defs["some_string_value"].default)
        self.assertEqual(30, parsed_variable_defs["some_int_value"].default)
        self.assertEqual(0.9, parsed_variable_defs["some_float_value"].default)
        self.assertIsNone(parsed_variable_defs["some_string_value_with_condition"].default)
        self.assertEqual(["a", "b"], parsed_variable_defs["some_list_of_string"].default)
        self.assertEqual({}, parsed_variable_defs["some_map_of_sring"].default)

        # Verify sensitive variable does not have default set
        self.assertIn("some_secret", parsed_variable_defs)
        self.assertIsNone(parsed_variable_defs["some_secret"].default)

        # Verify unrecognized variable from tfvars is not added
        self.assertNotIn("some_unrecognized_value", parsed_variable_defs)



import json
import os
from collections.abc import Generator
from pathlib import Path
from unittest.mock import patch

import pytest

from jupyter_deploy.engine.terraform.tf_open import TerraformOpenHandler


@pytest.fixture
def mock_cwd(tmp_path: Path) -> Generator[Path, None, None]:
    """Create a temporary directory and set it as the current working directory."""
    original_dir = os.getcwd()
    os.chdir(tmp_path)
    yield tmp_path
    os.chdir(original_dir)


@pytest.fixture
def mock_terraform_output() -> str:
    """Create a mock terraform output JSON string."""
    output_content = {"jupyter_url": {"value": "https://example.com/jupyter", "type": "string"}}
    return json.dumps(output_content)


class TestTerraformOpenHandler:
    def test_init(self) -> None:
        """Test that the TerraformOpenHandler initializes correctly."""
        handler = TerraformOpenHandler(project_path=Path("/fake/path"))
        assert handler.project_path == Path("/fake/path")

    def test_get_url_success(self, mock_terraform_output: str) -> None:
        """Test that get_url returns the Jupyter URL from the terraform output."""
        handler = TerraformOpenHandler(project_path=Path.cwd())
        with patch("jupyter_deploy.cmd_utils.run_cmd_and_capture_output", return_value=mock_terraform_output):
            url = handler.get_url()
            assert url == "https://example.com/jupyter"

    def test_get_url_no_output(self, mock_cwd: Path) -> None:
        """Test that get_url returns an empty string if there's no terraform output."""
        handler = TerraformOpenHandler(project_path=Path.cwd())
        with patch("jupyter_deploy.cmd_utils.run_cmd_and_capture_output", return_value="{}"):
            url = handler.get_url()
            assert url == ""

    def test_get_url_invalid_json(self, mock_cwd: Path) -> None:
        """Test that get_url raises a JSONDecodeError if the terraform output contains invalid JSON."""
        handler = TerraformOpenHandler(project_path=Path.cwd())
        with (
            patch("jupyter_deploy.cmd_utils.run_cmd_and_capture_output", return_value="invalid json"),
            pytest.raises(json.JSONDecodeError),
        ):
            handler.get_url()

    def test_get_url_missing_output(self, mock_cwd: Path) -> None:
        """Test that get_url returns an empty string if the terraform output doesn't contain the jupyter_url output."""
        output_content = {"other_output": {"value": "https://example.com/other", "type": "string"}}
        handler = TerraformOpenHandler(project_path=Path.cwd())
        with patch("jupyter_deploy.cmd_utils.run_cmd_and_capture_output", return_value=json.dumps(output_content)):
            url = handler.get_url()
            assert url == ""



import typer

servers_app = typer.Typer(
    help=("""Discover, start, stop or exec into server deployed to infrastructure controlled by your IaC templates."""),
    no_args_is_help=True,
)


@servers_app.command()
def list() -> None:
    """List servers running Jupyter notebooks from various Cloud Providers."""
    pass


@servers_app.command()
def describe() -> None:
    """Describe a server running a Jupyter notebook process."""
    pass



from abc import ABC, abstractmethod
from pathlib import Path

from jupyter_deploy.engine.enum import EngineType


class EngineDownHandler(ABC):
    def __init__(self, project_path: Path, engine: EngineType) -> None:
        """Instantiate the base handler for `jd down` command."""
        self.project_path = project_path
        self.engine = engine

    @abstractmethod
    def destroy(self, auto_approve: bool = False) -> None:
        pass



import unittest
from pathlib import Path
from unittest.mock import Mock, patch

from jupyter_deploy.engine.terraform.tf_variables import TerraformVariablesHandler


class TestTerraformVariablesHandler(unittest.TestCase):
    def test_successfully_instantiates(self) -> None:
        project_path = Path("/mock/project")
        handler = TerraformVariablesHandler(project_path=project_path)
        self.assertEqual(handler.project_path, project_path)

    @patch("jupyter_deploy.fs_utils.file_exists")
    def test_is_template_dir_return_true_when_variables_dot_tf_exists(self, mock_file_exists: Mock) -> None:
        mock_file_exists.return_value = True
        project_path = Path("/mock/project")
        handler = TerraformVariablesHandler(project_path=project_path)

        result = handler.is_template_directory()
        self.assertTrue(result)
        mock_file_exists.assert_called_once_with(project_path / "variables.tf")

    @patch("jupyter_deploy.fs_utils.file_exists")
    def test_is_template_dir_return_false_when_variables_dot_tf_does_not_exists(self, mock_file_exists: Mock) -> None:
        mock_file_exists.return_value = False
        project_path = Path("/mock/project")
        handler = TerraformVariablesHandler(project_path=project_path)

        result = handler.is_template_directory()
        self.assertFalse(result)
        mock_file_exists.assert_called_once_with(project_path / "variables.tf")

    @patch("jupyter_deploy.fs_utils.read_short_file")
    @patch("jupyter_deploy.engine.terraform.tf_varfiles.parse_variables_dot_tf_content")
    @patch("jupyter_deploy.engine.terraform.tf_varfiles.parse_dot_tfvars_content_and_add_defaults")
    def test_get_template_variables_returns_variables(
        self, mock_parse_tfvars: Mock, mock_parse_variables: Mock, mock_read_short_file: Mock
    ) -> None:
        project_path = Path("/mock/project")

        mock_read_short_file.side_effect = ["content-1", "content-2"]

        # mock terraform vars instances and their to_template_definition() method
        mock_1 = Mock()
        mock_2 = Mock()
        mock_3 = Mock()
        mock_to_template_def_1 = Mock()
        mock_to_template_def_2 = Mock()
        mock_to_template_def_3 = Mock()
        mock_1.to_template_definition = mock_to_template_def_1
        mock_2.to_template_definition = mock_to_template_def_2
        mock_3.to_template_definition = mock_to_template_def_3
        mock_to_template_def_1.return_value = {"val": "1"}
        mock_to_template_def_2.return_value = {"val": "2"}
        mock_to_template_def_3.return_value = {"val": "3"}

        # mock the variable parsing response
        mock_vars_from_vars_dot_tf = {"var1": mock_1, "var2": mock_2}
        mock_parse_variables.return_value = mock_vars_from_vars_dot_tf

        # make the tfvars modify the 2nd key value only
        def tfvars_side_effect(*largs, **kwargs) -> None:  # type: ignore
            input_vars = kwargs["variable_defs"]
            assert type(input_vars) is dict
            input_vars.update({"var2": mock_3})

        mock_parse_tfvars.side_effect = tfvars_side_effect

        # Act
        handler = TerraformVariablesHandler(project_path=project_path)
        result = handler.get_template_variables()

        # Assert
        self.assertEqual(result, {"var1": {"val": "1"}, "var2": {"val": "3"}})

        # should have read both vars.tf and .tfvars files
        self.assertEqual(mock_read_short_file.call_count, 2)
        self.assertEqual(mock_read_short_file.mock_calls[0][1][0], project_path / "variables.tf")
        self.assertEqual(mock_read_short_file.mock_calls[1][1][0], project_path / "defaults-all.tfvars")

        # should have parsed with the appropriate content
        mock_parse_variables.assert_called_once_with("content-1")
        mock_parse_tfvars.assert_called_once_with("content-2", variable_defs=mock_vars_from_vars_dot_tf)

        # only the final variable wrappers should have called their convert method
        mock_to_template_def_1.assert_called_once()
        mock_to_template_def_2.assert_not_called()
        mock_to_template_def_3.assert_called_once()

    @patch("jupyter_deploy.fs_utils.read_short_file")
    @patch("jupyter_deploy.engine.terraform.tf_varfiles.parse_variables_dot_tf_content")
    @patch("jupyter_deploy.engine.terraform.tf_varfiles.parse_dot_tfvars_content_and_add_defaults")
    def test_get_template_variables_raises_on_large_variables_dot_tf_file(
        self, mock_parse_tfvars: Mock, mock_parse_variables: Mock, mock_read_short_file: Mock
    ) -> None:
        # Prepare
        mock_read_short_file.side_effect = RuntimeError("File is too large!")

        # Act
        handler = TerraformVariablesHandler(project_path=Path("/mock/project"))
        with self.assertRaises(RuntimeError):
            handler.get_template_variables()

        # Verify
        mock_read_short_file.assert_called_once()
        mock_parse_tfvars.assert_not_called()
        mock_parse_variables.assert_not_called()

    @patch("jupyter_deploy.fs_utils.read_short_file")
    @patch("jupyter_deploy.engine.terraform.tf_varfiles.parse_variables_dot_tf_content")
    @patch("jupyter_deploy.engine.terraform.tf_varfiles.parse_dot_tfvars_content_and_add_defaults")
    def test_get_template_variables_raises_on_variables_dot_tf_read_error(
        self, mock_parse_tfvars: Mock, mock_parse_variables: Mock, mock_read_short_file: Mock
    ) -> None:
        # Prepare
        mock_read_short_file.side_effect = ["content-1", RuntimeError("File is too large!")]
        mock_parse_variables.return_value = {}

        # Act
        handler = TerraformVariablesHandler(project_path=Path("/mock/project"))
        with self.assertRaises(RuntimeError):
            handler.get_template_variables()

        # Verify
        mock_read_short_file.assert_called()
        mock_parse_variables.assert_called_once()
        mock_parse_tfvars.assert_not_called()

    @patch("jupyter_deploy.fs_utils.read_short_file")
    @patch("jupyter_deploy.engine.terraform.tf_varfiles.parse_variables_dot_tf_content")
    @patch("jupyter_deploy.engine.terraform.tf_varfiles.parse_dot_tfvars_content_and_add_defaults")
    def test_get_template_variables_raises_tfvars_read_error(
        self, mock_parse_tfvars: Mock, mock_parse_variables: Mock, mock_read_short_file: Mock
    ) -> None:
        # Prepare
        mock_read_short_file.side_effect = ["content-1", "content-2"]
        mock_1 = Mock()
        mock_to_template_def_1 = Mock()
        mock_1.to_template_definition = mock_to_template_def_1
        mock_parse_variables.return_value = {"val1": mock_1}
        mock_to_template_def_1.return_value = {"val": "1"}

        # Act
        handler = TerraformVariablesHandler(project_path=Path("/mock/project"))
        result1 = handler.get_template_variables()

        # Verify-1
        self.assertEqual(result1, {"val1": {"val": "1"}})
        self.assertEqual(mock_read_short_file.call_count, 2)
        self.assertEqual(mock_parse_variables.call_count, 1)
        self.assertEqual(mock_parse_tfvars.call_count, 1)
        self.assertEqual(mock_to_template_def_1.call_count, 1)

        # Act again
        result2 = handler.get_template_variables()
        self.assertEqual(result1, result2)
        self.assertEqual(mock_read_short_file.call_count, 2)
        self.assertEqual(mock_parse_variables.call_count, 1)
        self.assertEqual(mock_parse_tfvars.call_count, 1)
        self.assertEqual(mock_to_template_def_1.call_count, 1)



import unittest
from unittest.mock import Mock, patch

from packaging.version import Version

from jupyter_deploy.provider.aws import ssm_plugin


class TestCheckSsmPluginInstallation(unittest.TestCase):
    @patch("jupyter_deploy.cmd_utils.check_executable_installation")
    def test_returns_true_when_check_exec_installation(self, mock_check: Mock) -> None:
        # Mock the check_executable_installation to return successful installation
        mock_check.return_value = (True, "2.0.0", None)
        result = ssm_plugin.check_ssm_plugin_installation()

        # Verify
        self.assertTrue(result)
        mock_check.assert_called_once_with(executable_name="session-manager-plugin")

    @patch("jupyter_deploy.cmd_utils.check_executable_installation")
    def test_returns_false_when_check_exec_installation_not_found(self, mock_check: Mock) -> None:
        # Mock the check_executable_installation to return failed installation
        mock_check.return_value = (False, None, "Command 'session-manager-plugin' not found")
        result = ssm_plugin.check_ssm_plugin_installation()

        # Verify
        self.assertFalse(result)
        mock_check.assert_called_once()

    @patch("jupyter_deploy.cmd_utils.check_executable_installation")
    def test_returns_true_with_version_check(self, mock_check: Mock) -> None:
        # Mock the check_executable_installation to return successful installation with version
        mock_check.return_value = (True, "2.5.0", None)

        # Call the function under test with a minimum version requirement
        result = ssm_plugin.check_ssm_plugin_installation(min_version=Version("2.0.0"))

        # Verify
        self.assertTrue(result)
        mock_check.assert_called_once()

    @patch("jupyter_deploy.cmd_utils.check_executable_installation")
    def test_returns_false_when_version_check_fails(self, mock_check: Mock) -> None:
        mock_check.return_value = (True, "1.5.0", None)
        result = ssm_plugin.check_ssm_plugin_installation(min_version=Version("2.0.0"))

        # Verify
        self.assertFalse(result)
        mock_check.assert_called_once()

    @patch("jupyter_deploy.cmd_utils.check_executable_installation")
    def test_raises_when_check_exec_raises(self, mock_check: Mock) -> None:
        # Mock the check_executable_installation to raise an exception
        mock_check.side_effect = Exception("Test exception")
        with self.assertRaises(Exception) as context:
            ssm_plugin.check_ssm_plugin_installation()

        # Verify
        self.assertEqual(str(context.exception), "Test exception")
        mock_check.assert_called_once()



from abc import ABC, abstractmethod
from pathlib import Path

from jupyter_deploy.engine.enum import EngineType
from jupyter_deploy.engine.vardefs import TemplateVariableDefinition


class EngineVariablesHandler(ABC):
    def __init__(self, project_path: Path, engine: EngineType) -> None:
        """Instantiate the base handler for the decorator."""
        self.project_path = project_path
        self.engine = engine

    @abstractmethod
    def is_template_directory(self) -> bool:
        """Return True if the directory corresponds to a jupyter-deploy directory."""
        pass

    @abstractmethod
    def get_template_variables(self) -> dict[str, TemplateVariableDefinition]:
        """Return the dict of variable-name->variable-definition.

        This operation presumably requires file system operations and should
        be cached within each VariableHandler.
        """
        pass



from typing import Annotated, Literal

from pydantic import BaseModel, ConfigDict

from jupyter_deploy.engine.enum import EngineType


class JupyterDeployTemplateV1(BaseModel):
    model_config = ConfigDict(extra="allow")
    name: str
    engine: EngineType
    version: str


class JupyterDeployManifestV1(BaseModel):
    model_config = ConfigDict(extra="allow")
    schema_version: Literal[1]
    template: JupyterDeployTemplateV1

    def get_engine(self) -> EngineType:
        """Return the engine type."""
        return self.template.engine


# Combined type using discriminated union
JupyterDeployManifest = Annotated[JupyterDeployManifestV1, "schema_version"]



import json
from pathlib import Path

from rich import console as rich_console

from jupyter_deploy import cmd_utils
from jupyter_deploy.engine.engine_open import EngineOpenHandler
from jupyter_deploy.engine.terraform.tf_constants import TF_OUTPUT_CMD


class TerraformOpenHandler(EngineOpenHandler):
    """Terraform implementation of the EngineOpenHandler."""

    def __init__(self, project_path: Path) -> None:
        self.project_path = project_path

    def get_url(self) -> str:
        console = rich_console.Console()

        output_cmd = TF_OUTPUT_CMD.copy()

        output = cmd_utils.run_cmd_and_capture_output(output_cmd)
        output_dict = json.loads(output)

        if not output_dict:
            console.print(
                f":x: Terraform state file either has no outputs, or could not be found in {self.project_path}. "
                f"Have you run `jd up` from the project directory?",
                style="red",
            )
            return ""

        if "jupyter_url" not in output_dict or "value" not in output_dict["jupyter_url"]:
            console.print(
                ":x: Could not find jupyter_url value in Terraform state file. "
                "Have you run `jd up` from the project directory?",
                style="red",
            )
            return ""

        return str(output_dict["jupyter_url"]["value"])



from pathlib import Path

from jupyter_deploy import fs_utils
from jupyter_deploy.engine.engine_variables import EngineVariablesHandler
from jupyter_deploy.engine.terraform import tf_varfiles
from jupyter_deploy.engine.terraform.tf_constants import TF_VARIABLES_FILENAME, get_preset_filename
from jupyter_deploy.engine.vardefs import TemplateVariableDefinition


class TerraformVariablesHandler(EngineVariablesHandler):
    """Terraform-specific implementation of the VariableHandler."""

    def __init__(self, project_path: Path) -> None:
        self.project_path = project_path
        self._template_vars: dict[str, TemplateVariableDefinition] | None = None

    def is_template_directory(self) -> bool:
        return fs_utils.file_exists(self.project_path / TF_VARIABLES_FILENAME)

    def get_template_variables(self) -> dict[str, TemplateVariableDefinition]:
        # cache handling to avoid the expensive fs operation necessary
        # to retrieve the variable definitions.
        if self._template_vars:
            return self._template_vars

        # read the variables.tf, retrieve the description, sensitive
        variables_dot_tf_path = self.project_path / TF_VARIABLES_FILENAME
        variables_dot_tf_content = fs_utils.read_short_file(variables_dot_tf_path)
        variable_defs = tf_varfiles.parse_variables_dot_tf_content(variables_dot_tf_content)

        # read the template .tfvars with the defaults
        all_defaults_tfvars_path = self.project_path / get_preset_filename()
        variables_tfvars_content = fs_utils.read_short_file(all_defaults_tfvars_path)

        # combine
        tf_varfiles.parse_dot_tfvars_content_and_add_defaults(variables_tfvars_content, variable_defs=variable_defs)

        # translate to the engine-generic type
        template_vars = {var_name: var_def.to_template_definition() for var_name, var_def in variable_defs.items()}
        self._template_vars = template_vars
        return template_vars



"""Terraform implementation of the `down` handler."""

from pathlib import Path

from rich import console as rich_console

from jupyter_deploy import cmd_utils
from jupyter_deploy.engine.engine_down import EngineDownHandler
from jupyter_deploy.engine.enum import EngineType
from jupyter_deploy.engine.terraform.tf_constants import TF_AUTO_APPROVE_CMD_OPTION, TF_DESTROY_CMD


class TerraformDownHandler(EngineDownHandler):
    """Down handler implementation for terraform projects."""

    def __init__(self, project_path: Path) -> None:
        super().__init__(project_path=project_path, engine=EngineType.TERRAFORM)

    def destroy(self, auto_approve: bool = False) -> None:
        console = rich_console.Console()

        destroy_cmd = TF_DESTROY_CMD.copy()
        if auto_approve:
            destroy_cmd.append(TF_AUTO_APPROVE_CMD_OPTION)

        retcode, timed_out = cmd_utils.run_cmd_and_pipe_to_terminal(destroy_cmd)

        if retcode != 0 or timed_out:
            console.print(":x: Error destroying Terraform infrastructure.", style="red")
            return

        console.print("Infrastructure resources destroyed successfully.", style="green")



import webbrowser

from rich.console import Console

from jupyter_deploy.engine.engine_open import EngineOpenHandler
from jupyter_deploy.engine.enum import EngineType
from jupyter_deploy.engine.terraform import tf_open
from jupyter_deploy.handlers.base_project_handler import BaseProjectHandler


class OpenHandler(BaseProjectHandler):
    _handler: EngineOpenHandler

    def __init__(self) -> None:
        """Base class to manage the open command of a jupyter-deploy project."""
        super().__init__()
        self.console = Console()

        if self.engine == EngineType.TERRAFORM:
            self._handler = tf_open.TerraformOpenHandler(project_path=self.project_path)
        else:
            raise NotImplementedError(f"OpenHandler implementation not found for engine: {self.engine}")

    def open_url(self, url: str) -> None:
        """Launch the Jupyter URL in the default web browser."""
        if not url:
            return

        if not url.startswith("https://"):
            self.console.print(
                ":x: Insecure URL detected. Only HTTPS URLs are allowed for security reasons.",
                style="red",
            )
            return

        self.console.print(f"\nOpening Jupyter app at: {url}", style="green")
        self.console.print(
            "\n[yellow]Note:[/] If you're having trouble accessing the Jupyter notebook, "
            "you may need to clear your browser cookies for this domain.\n"
        )
        open_status = webbrowser.open(url, new=2)

        if not open_status:
            self.console.print(
                ":x: Failed to open URL in browser.",
                style="red",
            )

    def get_url(self) -> str:
        """Retrieve the Jupyter URL from the project state file outputs."""
        return self._handler.get_url()



import unittest

from typer.testing import CliRunner

from jupyter_deploy.cli.servers_app import servers_app


class TestServersApp(unittest.TestCase):
    """Test cases for the servers_app module."""

    def test_list_command(self) -> None:
        """Test the list command."""
        runner = CliRunner()
        result = runner.invoke(servers_app, ["list"])

        self.assertEqual(result.exit_code, 0, "list command should work")

        # Currently, the list command is empty, so we just verify it doesn't raise an exception
        # In the future, if this command is implemented, we should check its behavior

    def test_describe_command(self) -> None:
        """Test the describe command."""
        runner = CliRunner()
        result = runner.invoke(servers_app, ["describe"])

        self.assertEqual(result.exit_code, 0, "describe command should work")

        # Currently, the describe command is empty, so we just verify it doesn't raise an exception
        # In the future, if this command is implemented, we should check its behavior

    def test_help_command(self) -> None:
        """Test the help command."""
        self.assertTrue(len(servers_app.info.help or "") > 0, "help should not be empty")

        runner = CliRunner()
        result = runner.invoke(servers_app, ["--help"])

        self.assertEqual(result.exit_code, 0)
        self.assertTrue(result.stdout.index("list") > 0)
        self.assertTrue(result.stdout.index("describe") > 0)

    def test_no_arg_defaults_to_help(self) -> None:
        """Test that running the app with no arguments shows help."""
        runner = CliRunner()
        result = runner.invoke(servers_app, [])

        self.assertEqual(result.exit_code, 0)
        self.assertTrue(len(result.stdout) > 0)
